import asyncio
import os


class _Result():
    stdout = str
    stderr = str
    result = int

    def __init__(self, result, stdout, stderr, is_wine:bool=False):
        # Check values
        if not isinstance(result, int):
            raise ValueError("result have to be an int.")
        if not isinstance(stdout, str):
            raise ValueError("stdout have to be an string.")
        if not isinstance(stderr, str):
            raise ValueError("stderr have to be an string.")

        # Set values
        self.stdout = stdout
        self.stderr = stderr
        if is_wine:
            lines = stderr.splitlines(keepends=True)
            if len(lines) == 1 and lines[0].startswith("wine: cannot find '") and lines[0].endswith("'\n"):
                self.result = -1
            else:
                self.result = result
        else:
            self.result = result

    def __bool__(self):
        return self.result == 0
    
    def __repr__(self):
        return "<Wine-Result result=" + str(self.result) + " stdout=" + repr(self.stdout) + " stderr=" + repr(self.stderr) + ">"


class _RunCommand():
    __args = list
    __envs = dict
    __cwd: str

    def __init__(self, args, envs):
        self.__args = args
        self.__envs = envs
        self.__cwd = None

    @property
    def cwd(self):
        return self.__cwd

    @cwd.setter
    def cwd(self, cwd: str):
        if self.__cwd is not None:
            raise RuntimeError("change dir can be done onle ones.")
        if not isinstance(cwd, str):
            raise ValueError("change dir have to be a string.")
        self.__cwd = cwd

    async def run(self):
        # Copy own env
        env = dict(os.environ)
        for iID, i in self.__envs.items():
            env[iID] = i

        # Run proc
        proc = await asyncio.create_subprocess_exec(*(self.__args), stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE, env=env, cwd=self.__cwd)
        std_out, std_err = await proc.communicate()
        return _Result(proc.returncode, std_out.decode(), std_err.decode(), is_wine=self.__args[0] == b"wine")


class WineConfig():
    path = str
    home = str

    def __init__(self, path, home):
        # Check values
        if not isinstance(path, str):
            raise ValueError("path have to be a string.")
        if not isinstance(home, str):
            raise ValueError("home have to be a string.")

        # Set values
        self.path = path
        self.home = home

    def preapre_command(self, args, envs={}, with_display=True, cwd_in_instance: bool = False, cwd_path: str = None):
        envs = dict(envs)

        # Set base env
        envs["HOME"] = self.home
        envs["WINEPREFIX"] = self.path

        # Check if display have to remove
        if not with_display:
            envs["DISPLAY"] = ""
            envs["WAYLAND_DISPLAY"] = ""

        # Create prepared command
        result = _RunCommand(args, envs)

        # Change dir into instance
        if cwd_in_instance:
            result.cwd = os.path.join(self.path, "drive_c")
        if cwd_path:
            if cwd_in_instance:
                raise ValueError(
                    "cwd_path and cwd_in_instance can't be set at the same time.")
            result.cwd = cwd_path

        # Return result
        return result

    async def run_command(self, *args, **kargs):
        cmd = self.preapre_command(*args, **kargs)
        return await cmd.run()
    
    async def stop_wine(self):
        return await self.run_command([b"wineserver", b"-w"])
